-- @release $Id: Priority.lua 1 2016-08-13 18:29:06Z reighnman $

if not ShockAndAwe then return end

local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
if not C then
  ShockAndAwe:SetConstants()
end
local LBF = LibStub("LibButtonFacade", true)
local LBFgroup = nil
local _, _, _, clientVersion = GetBuildInfo()
local minaoetargets = 3
--if LBF then
-- local LBFgroup = LBF:Group("ShockAndAwe")
--end
ShockAndAwe.PriorityFrame = CreateFrame("Button", "SAA_PriorityFrame", UIParent)
ShockAndAwe.PriorityFrame.cooldown = CreateFrame("Cooldown", "SAA_PriorityFrameCooldown", ShockAndAwe.PriorityFrame, "CooldownFrameTemplate") --Inherit from WoW template (texture not necessary)

---------------------------
-- Local Variables
---------------------------

ShockAndAwe.priorityTable = {}
ShockAndAwe.priorityTable.name = {}
ShockAndAwe.priorityTable.icon = {}
ShockAndAwe.priorityTable.test = {}

-- table.foreach(ShockAndAwe.priorityTable.name, print)

--------------------------
-- Helper Functions
--------------------------
-- returns : true if spellname is available in less than priority.cooldown /run print(ShockAndAwe:SpellCooldown(ShockAndAwe.constants["Mortal Strike ID"]))
-- or if spellname is currently activated
-- , remaining cooldown, 0 if ready to be cast
function ShockAndAwe:SpellAvailable(spellid)
if ShockAndAwe.db.char.priority.hideImmune and self:IsTargetImmune(spellid) then
  return false
end
local start, duration = GetSpellCooldown(spellid)
if start then
  local timeleft = start + duration - GetTime()
  local gcd_duration = GetSpellCooldown(ShockAndAwe.constants["SpellCoolDown"]) - .25 --this is to account for lag currently in testing
  --if SlamAndAwe:IsOffGCD() then
  if timeleft < gcd_duration and timeleft <= ShockAndAwe.db.char.priority.cooldown then
    return true, duration, timeleft
    else
    return false, duration, timeleft
    -- else
    -- if timeleft < 0 then
    -- return true, duration, timeleft
    -- end
  end
end
return false, 999

end

function ShockAndAwe:SpellReady(spellid)
  if ShockAndAwe.db.char.priority.hideImmune and self:IsTargetImmune(spellid) then
    return false
  end
  local _, _, _, _, _, _, _, _, _, castid = UnitCastingInfo("player")
  local available = GetSpellInfo(GetSpellInfo(spellid))
  local usable = IsUsableSpell(spellid)
  local startTime, gcdDuration = GetSpellCooldown(ShockAndAwe.constants["SpellCoolDown"])
  local start, duration = GetSpellCooldown(spellid)
  if start then
    local timeleft = start + duration - GetTime()

    if ShockAndAwe.db.char.currentspec == "Elemental" then
      local gcdTime = startTime + gcdDuration - GetTime()
      if available and usable and castid ~= spellid and timeleft <= gcdTime then
        return true, usable
      end
    else
      if available and usable then
        return true, usable
      end
    end
  end
  return false, false
end

function ShockAndAwe:SpellCooldown(spellid) --/run print(GetSpellCooldown(ShockAndAwe.constants["Bladestorm ID"]))
  local start, duration = GetSpellCooldown(spellid)
  if start then
    local timeleft = start + duration - GetTime()
    return duration, timeleft
  end
  return false, 999
end

function ShockAndAwe:SpellCharges(spellid)
  local currentCharges, maxCharges, cooldownStart, cooldownDuration = GetSpellCharges(spellid)

  if maxCharges then
    local timeleft = cooldownStart + cooldownDuration - GetTime()
    local calc = (1-(timeleft/cooldownDuration))+currentCharges
    --local final = string.format("%.2f", calc)
    if currentCharges ~= maxCharges then
      return true, currentCharges, maxCharges, calc
    else
      return false, currentCharges, maxCharges, 0
    end
  else
    return false, 0, 0, 0
  end
end

function ShockAndAwe:PrintPriorities()
  for index = 1, 30 do
    ShockAndAwe:Print(ShockAndAwe.db.char.priority.prOption[index])
  end
end

function ShockAndAwe:IsBoss()
  local mylevel = UnitLevel("player")
  local bosslevel = mylevel + 3
  if UnitLevel("target") >= bosslevel or UnitClassification("target") == "worldboss" then
    return true
  end
end

function ShockAndAwe:GCDAvailable()
  local startTime, gcdDuration = GetSpellCooldown(ShockAndAwe.constants["SpellCoolDown"])
  local gcdTime = startTime + gcdDuration - GetTime()
  return gcdTime <= ShockAndAwe.db.char.priority.cooldown
end

function ShockAndAwe:IsOffGCD()
  local startTime, gcdDuration = GetSpellCooldown(ShockAndAwe.constants["SpellCoolDown"])
  local gcdTime = startTime + gcdDuration - GetTime()
  if gcdTime > 0 then
    return true
  else
    return false
  end
end

function ShockAndAwe:GetDebuffInfo(debuff)
  local index = 1
  while UnitDebuff("target", index) do
    local name, _, _, count, _, _, debuffExpires, unitCaster, _, _, id = UnitDebuff("target", index)
    local isMine = unitCaster == "player"
    if id == debuff and isMine then
      local duration = debuffExpires - GetTime()
      return duration > ShockAndAwe.db.char.priority.cooldown, duration, count, debuffExpires
    end
    index = index + 1
  end
  return false, 0, 0, 0
end

function ShockAndAwe:GetBuffInfo(buff)
  local index = 1
  while UnitBuff("player", index) do
    local name, _, _, count, _, _, buffExpires, _, _, _, id = UnitBuff("player", index)
    if id == buff then
      local duration = buffExpires - GetTime()
      return duration > ShockAndAwe.db.char.priority.cooldown, duration, count
    end
    index = index + 1
  end
  return false, 0, 0
end

function ShockAndAwe:IsTargetImmune(spell)
  local target = UnitName("target") or ""
  if spell and ShockAndAwe.db.char.immuneTargets[target.."_"..spell] then
    return true
  end
  return false
end

local minRange = 0
local items = {
  {5, 37727}, -- Ruby Acorn
  {6, 63427}, -- Worgsaw
  {8, 34368}, -- Attuned Crystal Cores /run print(IsItemInRange("Attuned Crystal Cores", "TARGET"))
  {10, 32321}, -- Sparrowhawk Net
  {15, 33069}, -- Sturdy Rope
  {20, 10645}, -- Gnomish Death Ray
  {25, 24268}, -- Netherweave Net
  {30, 835}, -- Large Rope Net
  {35, 24269}, -- Heavy Netherweave Net
  {40, 28767}, -- The Decapitator
  {45, 23836}, -- Goblin Rocket Launcher
  {50, 116139}, -- Haunting Memento
  {60, 32825}, -- Soul Cannon
  {70, 41265}, -- Eyesore Blaster
  {80, 35278}, -- Reinforced Net
  {100, 33119}, -- Malister's Frost Wand
}

function ShockAndAwe:CountNameplates(maxRange)
  local last = result or 0
  local counter = 0

  for i = 1, 40 do
    local unit = "nameplate"..i
    if UnitExists(unit) and not UnitIsFriend("player", unit) then
      local min, max = ShockAndAwe:GetRange(unit)
      if min and max and min >= minRange and max <= maxRange then
        counter = counter + 1
      end
    end
  end
  if counter > 0 then
    return counter
  else
    return 0
  end
end

function ShockAndAwe:GetRange(unit)
  if not UnitExists(unit) or UnitIsFriend("player", unit) then
    return nil
  end

  local minRange, maxRange = 0, 0
  for _, items in ipairs(items) do
    local range, item = items[1], items[2]
    if IsItemInRange("item:"..item, unit) then
      maxRange = range
      break
    end
    minRange = range
  end
  return minRange, maxRange
end

------------------
-- Misc Functions
------------------

function ShockAndAwe:EstimateIgnorePain()
  local versatilityDamageBonus = GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE) + GetVersatilityBonus(CR_VERSATILITY_DAMAGE_DONE);
  local versatilityDamangeBonusPerc = (1 + versatilityDamageBonus * 10 ^ -2)
  local curMaelstrom = UnitPower("player", "mana")

  if curMaelstrom >= 20 then
    local _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, currentIP = UnitBuff("Player","Ignore Pain")
    if not currentIP then
      currentIP = 0
    end
    local _, STR, _, _ = UnitStat("player", 1)
    local APbase, APposBuff, APnegBuff = UnitAttackPower("player")
    local APfinal = APbase + APposBuff + APnegBuff
    local ipMaxMaelstrom = 60
    return floor((7/15*min(curMaelstrom,ipMaxMaelstrom)*APfinal)*versatilityDamangeBonusPerc*ShockAndAwe.db.char.ipMultiplier*.9+currentIP)
  else
    return 0
  end
end

------------------
-- Priorities
------------------

function ShockAndAwe:SetPriorityTable()
  ShockAndAwe.priorityTable.name["none"] = L["None"]
  _, _, ShockAndAwe.priorityTable.icon["none"] = "Interface/Tooltips/UI-Tooltip-Background"
  ShockAndAwe.priorityTable.test["none"] = function () return false end
end

--Elemental
ShockAndAwe.priorityTable.name["lav"] = ShockAndAwe.constants["Lava Burst"]
ShockAndAwe.priorityTable.icon["lav"] = ShockAndAwe.constants["Lava Burst Icon"]
ShockAndAwe.priorityTable.test["lav"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Lava Burst ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lava Burst ID"])
  end
end

ShockAndAwe.priorityTable.name["lav_apl"] = "Lava Burst (if=dot.flame_shock.remains>cast_time&(cooldown_react|buff.ascendance.up))"
ShockAndAwe.priorityTable.icon["lav_apl"] = ShockAndAwe.constants["Lava Burst Icon"]
ShockAndAwe.priorityTable.test["lav_apl"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Lava Burst ID"]) then
    local FS, FSduration = ShockAndAwe:GetDebuffInfo(ShockAndAwe.constants["Flame Shock ID"])
    -- local LS = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Lava Surge Buff ID"])
    local _, _, _, castTime = GetSpellInfo(ShockAndAwe.constants["Lava Burst ID"])
    local realCastTime = castTime / 1000
    -- local asc, asc_dur = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Ascendance"])
    if FSduration > realCastTime then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lava Burst ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["fl_sh"] = "Flame Shock (if=!ticking)"
ShockAndAwe.priorityTable.icon["fl_sh"] = ShockAndAwe.constants["Flame Shock Icon"]
ShockAndAwe.priorityTable.test["fl_sh"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Flame Shock ID"]) then
    local FS, FSduration = ShockAndAwe:GetDebuffInfo(ShockAndAwe.constants["Flame Shock ID"])
    if not FS then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Flame Shock ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["fl_sh_asc"] = "Flame Shock (if=maelstrom>=20&remains<=buff.ascendance.duration&cooldown.ascendance.remains+buff.ascendance.duration<=duration)"
ShockAndAwe.priorityTable.icon["fl_sh_asc"] = ShockAndAwe.constants["Flame Shock Icon"]
ShockAndAwe.priorityTable.test["fl_sh_asc"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Flame Shock ID"]) then
    local maelstrom = UnitPower("player", "maelstrom")
    local asc, asc_dur = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Ascendance"])
    local FS, FSduration = ShockAndAwe:GetDebuffInfo(ShockAndAwe.constants["Flame Shock ID"])
    if maelstrom >= 20 and FSduration <= asc_dur then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Flame Shock ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["el_bl"] = ShockAndAwe.constants["Elemental Blast"]
ShockAndAwe.priorityTable.icon["el_bl"] = ShockAndAwe.constants["Elemental Blast Icon"]
ShockAndAwe.priorityTable.test["el_bl"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Elemental Blast ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Elemental Blast ID"])
  end
end

ShockAndAwe.priorityTable.name["ea_sh92"] = "Earth Shock (if=maelstrom >= 92)"
ShockAndAwe.priorityTable.icon["ea_sh92"] = ShockAndAwe.constants["Earth Shock Icon"]
ShockAndAwe.priorityTable.test["ea_sh92"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Earth Shock ID"]) then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 92 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Earth Shock ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ea_sh86"] = "Earth Shock (if=maelstrom >= 86)"
ShockAndAwe.priorityTable.icon["ea_sh86"] = ShockAndAwe.constants["Earth Shock Icon"]
ShockAndAwe.priorityTable.test["ea_sh86"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Earth Shock ID"]) then
    local maelstrom = UnitPower("player", "mana")
    if maelstrom >= 86 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Earth Shock ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ea_sh"] = ShockAndAwe.constants["Earth Shock"]
ShockAndAwe.priorityTable.icon["ea_sh"] = ShockAndAwe.constants["Earth Shock Icon"]
ShockAndAwe.priorityTable.test["ea_sh"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Earth Shock ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Earth Shock ID"])
  end
end

ShockAndAwe.priorityTable.name["if"] = ShockAndAwe.constants["Ice Fury"]
ShockAndAwe.priorityTable.icon["if"] = ShockAndAwe.constants["Ice Fury Icon"]
ShockAndAwe.priorityTable.test["if"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Ice Fury ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Ice Fury ID"])
  end
end

--Enhancement

ShockAndAwe.priorityTable.name["fs"] = ShockAndAwe.constants["Feral Spirit"]
ShockAndAwe.priorityTable.icon["fs"] = ShockAndAwe.constants["Feral Spirit Icon"]
ShockAndAwe.priorityTable.test["fs"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Feral Spirit ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Feral Spirit ID"])
  end
end

ShockAndAwe.priorityTable.name["fs_boss"] = "Feral Spirit (if=boss)"
ShockAndAwe.priorityTable.icon["fs_boss"] = ShockAndAwe.constants["Feral Spirit Icon"]
ShockAndAwe.priorityTable.test["fs_boss"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Feral Spirit ID"]) and ShockAndAwe:IsBoss() then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Feral Spirit ID"])
  end
end

ShockAndAwe.priorityTable.name["bf_nbf"] = "Boulderfist (if=!buff.boulderfist)"
ShockAndAwe.priorityTable.icon["bf_nbf"] = ShockAndAwe.constants["Boulderfist Icon"]
ShockAndAwe.priorityTable.test["bf_nbf"] =
function ()
  if ShockAndAwe.db.char.talBoulderfist then
    if ShockAndAwe:SpellReady(ShockAndAwe.constants["Boulderfist ID"]) then
      local BF = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Boulderfist Buff ID"])
      if not BF then
        return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Boulderfist ID"])
      end
    end
  end
end

ShockAndAwe.priorityTable.name["bf2"] = "Boulderfist (if=buff.boulderfist < gcd | boulderfist.charges > 1.75)"
ShockAndAwe.priorityTable.icon["bf2"] = ShockAndAwe.constants["Boulderfist Icon"]
ShockAndAwe.priorityTable.test["bf2"] =
function ()
  if ShockAndAwe.db.char.talBoulderfist then
    if ShockAndAwe:SpellReady(ShockAndAwe.constants["Boulderfist ID"]) then
      local hasCharges, currentCharges, maxCharges, fracCharges = ShockAndAwe:SpellCharges(ShockAndAwe.constants["Boulderfist ID"])
      local BF = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Boulderfist Buff ID"])
      if fracCharges > 1.75 or not BF then
        return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Boulderfist ID"])
      end
    end
  end
end

ShockAndAwe.priorityTable.name["bf"] = ShockAndAwe.constants["Boulderfist"]
ShockAndAwe.priorityTable.icon["bf"] = ShockAndAwe.constants["Boulderfist Icon"]
ShockAndAwe.priorityTable.test["bf"] =
function ()
  if ShockAndAwe.db.char.talBoulderfist then
    if ShockAndAwe:SpellReady(ShockAndAwe.constants["Boulderfist ID"]) then
      local charges = GetSpellCharges(ShockAndAwe.constants["Boulderfist ID"])
      if charges >= 1 then
        return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Boulderfist ID"])
      end
    end
  end
end

ShockAndAwe.priorityTable.name["rb"] = ShockAndAwe.constants["Rockbiter"]
ShockAndAwe.priorityTable.icon["rb"] = ShockAndAwe.constants["Rockbiter Icon"]
ShockAndAwe.priorityTable.test["rb"] =
function ()
  if not ShockAndAwe.db.char.talBoulderfist then
    if ShockAndAwe:SpellReady(ShockAndAwe.constants["Rockbiter ID"]) then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Rockbiter ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["asc"] = ShockAndAwe.constants["Ascendance"]
ShockAndAwe.priorityTable.icon["asc"] = ShockAndAwe.constants["Ascendance Icon"]
ShockAndAwe.priorityTable.test["asc"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Ascendance ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Ascendance ID"])
  end
end

ShockAndAwe.priorityTable.name["asc_boss"] = "Ascendance (if=boss)"
ShockAndAwe.priorityTable.icon["asc_boss"] = ShockAndAwe.constants["Ascendance Icon"]
ShockAndAwe.priorityTable.test["asc_boss"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Ascendance ID"]) and ShockAndAwe:IsBoss() then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Ascendance ID"])
  end
end

ShockAndAwe.priorityTable.name["ws"] = ShockAndAwe.constants["Windsong"]
ShockAndAwe.priorityTable.icon["ws"] = ShockAndAwe.constants["Windsong Icon"]
ShockAndAwe.priorityTable.test["ws"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Windsong ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Windsong ID"])
  end
end

ShockAndAwe.priorityTable.name["foa_nfoa"] = "Fury of Air (if=!buff.furyofair and maelstrom >= 10)"
ShockAndAwe.priorityTable.icon["foa_nfoa"] = ShockAndAwe.constants["Fury of Air Icon"]
ShockAndAwe.priorityTable.test["foa_nfoa"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Fury of Air ID"]) then
    local FA = UnitAura("player", GetSpellInfo(ShockAndAwe.constants["Fury of Air ID"]))
    local maelstrom = UnitPower("player", "mana")
    if not FA and maelstrom >= 10 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Fury of Air ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["fb"] = ShockAndAwe.constants["Frostbrand"]
ShockAndAwe.priorityTable.icon["fb"] = ShockAndAwe.constants["Frostbrand Icon"]
ShockAndAwe.priorityTable.test["fb"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Frostbrand ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Frostbrand ID"])
  end
end

ShockAndAwe.priorityTable.name["fb_hs"] = "Frostbrand (if=tal.hailstorm&buff.frostbrand < 4.8)"
ShockAndAwe.priorityTable.icon["fb_hs"] = ShockAndAwe.constants["Frostbrand Icon"]
ShockAndAwe.priorityTable.test["fb_hs"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Frostbrand ID"]) then
    local fb, fb_dur, fb_count = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Frostbrand ID"])
    if ShockAndAwe.db.char.talHailstorm and fb_dur < 4.8 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Frostbrand ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["fb_nfb"] = "Frostbrand (if=tal.hailstorm&buff.frostbrand < gcd)"
ShockAndAwe.priorityTable.icon["fb_nfb"] = ShockAndAwe.constants["Frostbrand Icon"]
ShockAndAwe.priorityTable.test["fb_nfb"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Frostbrand ID"]) then
    local fb, fb_dur, fb_count = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Frostbrand ID"])
    if ShockAndAwe.db.char.talHailstorm and not fb then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Frostbrand ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ft"] = ShockAndAwe.constants["Flametongue"]
ShockAndAwe.priorityTable.icon["ft"] = ShockAndAwe.constants["Flametongue Icon"]
ShockAndAwe.priorityTable.test["ft"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Flametongue ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Flametongue ID"])
  end
end

ShockAndAwe.priorityTable.name["ft_nft"] = "Flametongue(if=buff.flametongue < gcd)"
ShockAndAwe.priorityTable.icon["ft_nft"] = ShockAndAwe.constants["Flametongue Icon"]
ShockAndAwe.priorityTable.test["ft_nft"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Flametongue ID"]) then
    local ft, ft_dur, ft_count = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Flametongue Buff ID"])
    if not ft then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Flametongue ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ft_dur"] = "Flametongue (if=buff.flametongue < 4.8)"
ShockAndAwe.priorityTable.icon["ft_dur"] = ShockAndAwe.constants["Flametongue Icon"]
ShockAndAwe.priorityTable.test["ft_dur"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Flametongue ID"]) then
    local ft, ft_dur, ft_count = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Flametongue Buff ID"])
    if ft_dur < 4.8 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Flametongue ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ft_bf"] = "Flametongue (if=tal.boulderfist)"
ShockAndAwe.priorityTable.icon["ft_bf"] = ShockAndAwe.constants["Flametongue Icon"]
ShockAndAwe.priorityTable.test["ft_bf"] =
function ()
  if ShockAndAwe.db.char.talBoulderfist then
    if ShockAndAwe:SpellReady(ShockAndAwe.constants["Flametongue ID"]) then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Flametongue ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["dw"] = ShockAndAwe.constants["Doom Winds"]
ShockAndAwe.priorityTable.icon["dw"] = ShockAndAwe.constants["Doom Winds Icon"]
ShockAndAwe.priorityTable.test["dw"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Doom Winds ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Doom Winds ID"])
  end
end

ShockAndAwe.priorityTable.name["cl"] = ShockAndAwe.constants["Crash Lightning"]
ShockAndAwe.priorityTable.icon["cl"] = ShockAndAwe.constants["Crash Lightning Icon"]
ShockAndAwe.priorityTable.test["cl"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Crash Lightning ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Crash Lightning ID"])
  end
end

ShockAndAwe.priorityTable.name["cl_aoe"] = "Crash Lightning (if=adds >= 3)"
ShockAndAwe.priorityTable.icon["cl_aoe"] = ShockAndAwe.constants["Crash Lightning Icon"]
ShockAndAwe.priorityTable.test["cl_aoe"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Crash Lightning ID"]) then
    local Adds = ShockAndAwe:CountNameplates(8)
    if Adds >= minaoetargets then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Crash Lightning ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["cl_apl"] = "Crash Lightning (if=tal.crashingstorm or adds >= 2 or fs.remaining > 5)"
ShockAndAwe.priorityTable.icon["cl_apl"] = ShockAndAwe.constants["Crash Lightning Icon"]
ShockAndAwe.priorityTable.test["cl_apl"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Crash Lightning ID"]) then
    local fs, fs_cd = ShockAndAwe:SpellCooldown(ShockAndAwe.constants["Feral Spirit ID"])
    local Adds = ShockAndAwe:CountNameplates(8)
    if Adds >= 2 or ShockAndAwe.db.char.talCrashingStorm or fs_cd > 110 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Crash Lightning ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ws"] = ShockAndAwe.constants["Windstrike"]
ShockAndAwe.priorityTable.icon["ws"] = ShockAndAwe.constants["Windstrike Icon"]
ShockAndAwe.priorityTable.test["ws"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Windstrike ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Windstrike ID"])
  end
end

ShockAndAwe.priorityTable.name["ss"] = ShockAndAwe.constants["Stormstrike"]
ShockAndAwe.priorityTable.icon["ss"] = ShockAndAwe.constants["Stormstrike Icon"]
ShockAndAwe.priorityTable.test["ss"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Stormstrike ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Stormstrike ID"])
  end
end

ShockAndAwe.priorityTable.name["lb"] = ShockAndAwe.constants["Lightning Bolt"]
ShockAndAwe.priorityTable.icon["lb"] = ShockAndAwe.constants["Lightning Bolt Icon"]
ShockAndAwe.priorityTable.test["lb"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Lightning Bolt ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lightning Bolt ID"])
  end
end

ShockAndAwe.priorityTable.name["lb_oc"] = "Lightning Bolt (if=tal.overcharge and maelstrom >= 45)"
ShockAndAwe.priorityTable.icon["lb_oc"] = ShockAndAwe.constants["Lightning Bolt Icon"]
ShockAndAwe.priorityTable.test["lb_oc"] =
function ()
  if ShockAndAwe.db.char.talOvercharge then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 45 and ShockAndAwe:SpellReady(ShockAndAwe.constants["Lightning Bolt ID"]) then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lightning Bolt ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["lb_oc60"] = "Lightning Bolt (if=tal.overcharge and maelstrom >= 60)"
ShockAndAwe.priorityTable.icon["lb_oc60"] = ShockAndAwe.constants["Lightning Bolt Icon"]
ShockAndAwe.priorityTable.test["lb_oc60"] =
function ()
  if ShockAndAwe.db.char.talOvercharge then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 60 and ShockAndAwe:SpellReady(ShockAndAwe.constants["Lightning Bolt ID"]) then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lightning Bolt ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ll"] = ShockAndAwe.constants["Lava Lash"]
ShockAndAwe.priorityTable.icon["ll"] = ShockAndAwe.constants["Lava Lash Icon"]
ShockAndAwe.priorityTable.test["ll"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Lava Lash ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lava Lash ID"])
  end
end

ShockAndAwe.priorityTable.name["ll_hh"] = "Lava Lash (if=tal.hothand and hothand.react)"
ShockAndAwe.priorityTable.icon["ll_hh"] = ShockAndAwe.constants["Lava Lash Icon"]
ShockAndAwe.priorityTable.test["ll_hh"] =
function ()
  if ShockAndAwe.db.char.talHotHand then
    local hh = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Hot Hand ID"])
    if hh and ShockAndAwe:SpellReady(ShockAndAwe.constants["Lava Lash ID"]) then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lava Lash ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ll_90"] = "Lava Lash (if=maelstrom >= 90"
ShockAndAwe.priorityTable.icon["ll_90"] = ShockAndAwe.constants["Lava Lash Icon"]
ShockAndAwe.priorityTable.test["ll_90"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Lava Lash ID"]) then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 90 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lava Lash ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["ll_120"] = "Lava Lash (if=maelstrom >= 120"
ShockAndAwe.priorityTable.icon["ll_120"] = ShockAndAwe.constants["Lava Lash Icon"]
ShockAndAwe.priorityTable.test["ll_120"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Lava Lash ID"]) then
    local maelstrom = UnitPower("player", "maelstrom")
    if maelstrom >= 120 then
      return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Lava Lash ID"])
    end
  end
end

ShockAndAwe.priorityTable.name["es"] = ShockAndAwe.constants["Earthen Spike"]
ShockAndAwe.priorityTable.icon["es"] = ShockAndAwe.constants["Earthen Spike Icon"]
ShockAndAwe.priorityTable.test["es"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Earthen Spike ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Earthen Spike ID"])
  end
end

ShockAndAwe.priorityTable.name["sun"] = ShockAndAwe.constants["Sundering"]
ShockAndAwe.priorityTable.icon["sun"] = ShockAndAwe.constants["Sundering Icon"]
ShockAndAwe.priorityTable.test["sun"] =
function ()
  if ShockAndAwe:SpellReady(ShockAndAwe.constants["Sundering ID"]) then
    return ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Sundering ID"])
  end
end

-------------------------------
-- Priority Frame functions
-------------------------------

function ShockAndAwe:SetPriorityBackdrop(icon)

  if ShockAndAwe.db.char.priority.enableDisableBorder then

    self.PriorityFrame:SetBackdrop({ bgFile = icon
      })
  else
    --self.PriorityFrame.Icon:Set(icon)

    self.PriorityFrame:SetBackdrop({ bgFile = icon,
        edgeFile = media:Fetch("border", ShockAndAwe.db.char.priority.barborder),
        tile = false, tileSize = 0, edgeSize = 12,
        insets = { left = 2, right = 2, top = 2, bottom = 2 }
      })
    --[[
    if ShockAndAwe.db.char.currentspec == "Enhancement" then
      if ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Enmaelstrom ID"]) then
        self.PriorityFrame:SetBackdropBorderColor(1,0,0,1)
      else
        self.PriorityFrame:SetBackdropBorderColor(1,1,1,1)
      end
    end
    ]]
  end
end

function ShockAndAwe:CreatePriorityFrame()
  self.updatePriority = false
  self.PriorityFrame:SetScale(ShockAndAwe.db.char.priority.scale)
  self.PriorityFrame:SetFrameStrata(ShockAndAwe.db.char.framestratalevel)
  self.PriorityFrame:SetWidth(ShockAndAwe.db.char.priority.fWidth)
  self.PriorityFrame:SetHeight(ShockAndAwe.db.char.priority.fHeight)
  self.PriorityFrame.Icon = _G["SAA_PriorityFrameIcon"]
  self:SetPriorityBackdrop("Interface/Tooltips/UI-Tooltip-Background")
  self.PriorityFrame:SetBackdropColor(0, 0, 0, ShockAndAwe.db.char.priority.alpha);
  self.PriorityFrame:SetMovable(true);
  self.PriorityFrame:RegisterForDrag("LeftButton");
  self.PriorityFrame:EnableMouse(false)
  self.PriorityFrame:SetPoint(ShockAndAwe.db.char.priority.point, ShockAndAwe.db.char.priority.relativeTo, ShockAndAwe.db.char.priority.relativePoint, ShockAndAwe.db.char.priority.xOffset, ShockAndAwe.db.char.priority.yOffset)
  self.PriorityFrame:SetScript("OnDragStart",
    function()
      self.PriorityFrame:StartMoving();
    end );
  self.PriorityFrame:SetScript("OnDragStop",
    function()
      self.PriorityFrame:StopMovingOrSizing();
      self.PriorityFrame:SetScript("OnUpdate", nil);
      self:FinishedMoving(ShockAndAwe.db.char.priority, self.PriorityFrame);
    end );
  self:SetPriorityUpdateScript()
  if not self.PriorityFrame.topText then
    self.PriorityFrame.topText = self.PriorityFrame:CreateFontString(nil, "OVERLAY")
  end
  self.PriorityFrame.topText:SetTextColor(1,1,1,1)
  self.PriorityFrame.topText:SetFont(media:Fetch("font", ShockAndAwe.db.char.barfont), ShockAndAwe.db.char.barfontsize)
  self.PriorityFrame.topText:SetPoint("TOP", self.PriorityFrame, "TOP", 0, ShockAndAwe.db.char.barfontsize + 2)
  self.PriorityFrame.topText:SetText(string.format(L["Next Priority (Set %s)"], ShockAndAwe.db.char.priority.groupnumber))
  self.PriorityFrame.cooldown:SetAllPoints(self.PriorityFrame)
  --self.PriorityFrame.cooldown:SetSwipeTexture('Interface/Cooldown/edge')
  -- self.PriorityFrame.cooldown:SetSwipeColor(0, 0, 0, 0.8)
  self:CreateInterruptPurgeFrames()

  if ShockAndAwe.db.char.priority.show and InCombatLockdown() then
    self.PriorityFrame:Show()
  else
    self.PriorityFrame:Hide()
  end
  if ShockAndAwe.db.char.priority.titleshow then
    self.PriorityFrame.topText:Show()
  else
    self.PriorityFrame.topText:Hide()
  end
  if LBF and LBFgroup then
    LBFgroup:AddButton(self.PriorityFrame)
  end
end

function ShockAndAwe:ResetPriority()
  self.PriorityFrame:ClearAllPoints()
  ShockAndAwe.db.char.priority.point = self.defaults.char.priority.point
  ShockAndAwe.db.char.priority.relativeTo = self.defaults.char.priority.relativeTo
  ShockAndAwe.db.char.priority.relativePoint = self.defaults.char.priority.relativePoint
  ShockAndAwe.db.char.priority.xOffset = self.defaults.char.priority.xOffset
  ShockAndAwe.db.char.priority.yOffset = self.defaults.char.priority.yOffset
  ShockAndAwe.db.char.priority.fWidth = self.defaults.char.priority.fWidth
  ShockAndAwe.db.char.priority.fHeight = self.defaults.char.priority.fHeight
  ShockAndAwe.db.char.priority.scale = self.defaults.char.priority.scale
  ShockAndAwe.db.char.priority.prOption = self.defaults.char.priority.prOption
  self.PriorityFrame:SetPoint(ShockAndAwe.db.char.priority.point, ShockAndAwe.db.char.priority.relativeTo, ShockAndAwe.db.char.priority.relativePoint, ShockAndAwe.db.char.priority.xOffset, ShockAndAwe.db.char.priority.yOffset)
  self:CreatePriorityFrame()
  self:Print(L["priority_reset"])
end

function ShockAndAwe:SetPriorityUpdateScript()
  self.PriorityFrame:SetScript("OnUpdate",
    function()

      if self.updatePriority then
        ShockAndAwe.db.char.priority.previous = ShockAndAwe.db.char.priority.next
        ShockAndAwe:SetNextPriority()
        -- if ShockAndAwe.db.char.priority.next ~= ShockAndAwe.db.char.priority.previous or ShockAndAwe:CheckShamanisticMaelstrom() then
        ShockAndAwe:SetPriorityIcon(ShockAndAwe.db.char.priority.next) -- always set icon to fix the times where it was getting stuck
        -- end
        if ShockAndAwe.db.char.priority.showcooldown then
          local startTime, duration, enabled = GetSpellCooldown(ShockAndAwe.constants["SpellCoolDown"])
          if startTime then
            -- ShockAndAwe:DebugPrint("Cooldown "..startTime.. " - "..duration)

            ShockAndAwe.PriorityFrame.cooldown:SetCooldown(startTime, duration)
          end
        end
      end

      if ShockAndAwe.db.char.priority.show and not ShockAndAwe.db.char.disabled then
        if ShockAndAwe.db.char.priority.showinterrupt and self.PriorityFrame.interrupt then
          local EnemySpell, _, _, EnemySpellIcon, _, _, _, _, EnemySpellInterruptable = UnitCastingInfo("target")
          EnemySpellInterruptable = not EnemySpellInterruptable --reverse since false actually means interruptable
          if not EnemySpell then
            -- OBU.EnemySpellIcon = "Interface/Tooltips/UI-Tooltip-Background"
            self.PriorityFrame.interrupt.frame:SetBackdropColor(0, 0, 0, 0);
          end

          --This section causes high memory allocation?
          self.PriorityFrame.interrupt.frame:SetBackdrop({ bgFile = EnemySpellIcon,
              edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
              tile = false, tileSize = 0, edgeSize = 12,
              insets = { left = 2, right = 2, top = 2, bottom = 2 }
            })
          --------

          self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 1, 1, 1);

          local usable, nomana = IsUsableSpell(ShockAndAwe.constants["Wind Shear ID"]);
          local available = GetSpellCooldown(GetSpellInfo(ShockAndAwe.constants["Wind Shear ID"]))
          if EnemySpellInterruptable then
            if (usable and available == 0) then
              self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(0, 1, 0, 1);
              -- self:SetSubFrameBackdrop(self.PriorityFrame.interrupt.frame, OBU.EnemySpellIcon, 4)
              -- self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1,0,0,1)
            else
              self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 0, 0, 1);
              --self.PriorityFrame.purge.frame:SetBackdropColor(0, 0, 0, 0);
              --self:SetSubFrameBackdrop(self.PriorityFrame.interrupt.frame, OBU.EnemySpellIcon, 12)
              --self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(1, 0, 0, 1); -- set border color to Red if Purge not available
            end
          end
          if not EnemySpell then
            self.PriorityFrame.interrupt.frame:SetBackdropColor(0, 0, 0, 0);
            self.PriorityFrame.interrupt.frame:SetBackdropBorderColor(0, 0, 0, 0);
          end
        end
      end

    end );
end

function ShockAndAwe:SetPriorityIcon(priority)
  --ShockAndAwe:DebugPrint("setting icon to "..priority)
  if InCombatLockdown() or priority == "none" then
    local icon = self.priorityTable.icon[priority]
    -- if self:CheckWindShear() then
    -- self:SetSubFrameBackdrop(self.PriorityFrame.purge.frame, ShockAndAwe.constants["Heroic Strike Icon"], 4)
    -- else
    -- self:SetSubFrameBackdrop(self.PriorityFrame.purge.frame, "Interface/Tooltips/UI-Tooltip-Background", 4)
    -- self.PriorityFrame.purge.frame:SetBackdropColor(0, 0, 0, 0);
    -- end
    self:SetPriorityBackdrop(icon)
  end
end

function ShockAndAwe:CheckWindShear()
  if ShockAndAwe.db.char.hsshow then
    -- local UL = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Ultimatum"])
    -- if UL then
    -- return true
    -- else
    -- local avail = ShockAndAwe:SpellAvailable(ShockAndAwe.constants["Heroic Strike"])
    -- local usable, nomana = IsUsableSpell(ShockAndAwe.constants["Heroic Strike"])
    -- if usable and avail then
    -- local TargetSubTwenty = ShockAndAwe:TargetSubTwenty()
    -- local CS, CSduration = ShockAndAwe:GetDebuffInfo(ShockAndAwe.constants["Colossus Smash"])
    -- local TFB, TFBduration, TFBcount = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Taste for Blood"])
    -- local BC = ShockAndAwe:GetBuffInfo(ShockAndAwe.constants["Battle Cry"])
    -- local maelstrom = UnitPower("player", "mana")

    -- if maelstrom >= ShockAndAwe.db.char.maelstromThreshold then
    -- return true
    -- elseif ShockAndAwe.db.char.currentspec == "Enhancement" then
    --Enhancement
    -- if not TargetSubTwenty then
    -- -- Target below 20% health
    -- if CS then
    -- if maelstrom >= 40 then
    -- return true
    -- end
    -- elseif (DC and CS) then
    -- if maelstrom >= 30 then
    -- return true
    -- end
    -- end
    -- end
    --End Enhancement
    -- elseif ShockAndAwe.db.char.currentspec == "Arms" then
    --Arms
    -- if not TargetSubTwenty then
    -- if ((TFB and TFBduration <=2) or (TFBcount == 5) or (TFB and CSduration <= 2) or BC) and CS then
    -- return true
    -- end
    -- end
    -- end
    -- end
    -- end
  end
end

function ShockAndAwe:SetNextPriority()
  -- force update of stacks count to ensure priority shown correctly
  -- ShockAndAwe.db.char.msstacks = ShockAndAwe:GetMaelstromInfo()
  -- if ShockAndAwe.db.char.priority.combopoints then
  -- self:ShowComboPoints()
  -- end

  ShockAndAwe.db.char.priority.next = "none"
  for index = 1, 30 do
    if ShockAndAwe.db.char.priority.prOption[index] and self.priorityTable.test[ShockAndAwe.db.char.priority.prOption[index]] then -- verify that the option actually exists
      if self.priorityTable.test[ShockAndAwe.db.char.priority.prOption[index]]() then
        ShockAndAwe.db.char.priority.next = ShockAndAwe.db.char.priority.prOption[index]
        return -- we need to break out of the routine as we have found the top priority
      end
    end
  end
end

function ShockAndAwe:TargetSubTwenty()
  local dechp
  local perchp
  local calchp
  local currenthp = UnitHealth("target")
  local totalhp = UnitHealthMax("target")
  if currenthp == 0 then
    dechp = 0
    perchp = 0
  elseif currenthp == totalhp then
    perchp = 100
  else
    calchp = currenthp / totalhp
    dechp = calchp * 100
    perchp = math.ceil(dechp)
  end

  if perchp <= 20 then
    return true
  else
    return false
  end
end

function ShockAndAwe:SelfSubTwenty()
  local selfcurrenthp = UnitHealth("player")
  local selftotalhp = UnitHealthMax("player")
  local selfdechp
  local selfperchp
  local selfcalchp
  if selfcurrenthp == 0 then
    selfdechp = 0
    selfperchp = 0
  elseif currenthp == totalhp then
    selfperchp = 100
  else
    local selfcalchp = selfcurrenthp / selftotalhp
    local selfdechp = selfcalchp * 100
    selfperchp = math.ceil(dechp)
  end

  if selfperchp <= 20 then
    return true
  else
    return false
  end
end

function ShockAndAwe:CreateInterruptPurgeFrames()

  local width = ShockAndAwe.db.char.priority.fWidth / 1.6
  local height = ShockAndAwe.db.char.priority.fHeight / 1.6

  if not self.PriorityFrame.interrupt then
    self.PriorityFrame.interrupt = {}
    self.PriorityFrame.interrupt.frame = CreateFrame("Frame", "SAA_PriorityInterruptFrame", self.PriorityFrame)
  end
  local interruptFrame = self.PriorityFrame.interrupt.frame
  interruptFrame:SetScale(ShockAndAwe.db.char.priority.scale)
  interruptFrame:SetFrameStrata("BACKGROUND")
  interruptFrame:SetWidth(width)
  interruptFrame:SetHeight(height)

  --ShockAndAwe:SetTexture("Interface/Tooltips/UI-Tooltip-Background", false)

  self:SetSubFrameBackdrop(interruptFrame, "Interface/Tooltips/UI-Tooltip-Background", 4)
  interruptFrame:SetBackdropColor(0, 0, 0, 0);
  interruptFrame:SetPoint("BOTTOMLEFT", self.PriorityFrame, "BOTTOMRIGHT", 0 , 0)
  if ShockAndAwe.db.char.priority.showinterrupt then
    interruptFrame:Show()
  else
    interruptFrame:Hide()
  end

  if not self.PriorityFrame.purge then
    self.PriorityFrame.purge = {}
    self.PriorityFrame.purge.frame = CreateFrame("Frame", "SAA_PrioritypurgeFrame", self.PriorityFrame)
  end

  local purgeFrame = self.PriorityFrame.purge.frame
  purgeFrame:SetScale(ShockAndAwe.db.char.priority.scale)
  purgeFrame:SetFrameStrata("BACKGROUND")
  purgeFrame:SetWidth(width)
  purgeFrame:SetHeight(height)
  self:SetSubFrameBackdrop(purgeFrame, "Interface/Tooltips/UI-Tooltip-Background", 4)
  purgeFrame:SetBackdropColor(0, 0, 0, 0);
  purgeFrame:SetPoint("BOTTOMLEFT", self.PriorityFrame, "BOTTOMRIGHT", 0 , 0)
  if ShockAndAwe.db.char.priority.showpurge then
    purgeFrame:Show()
  else
    purgeFrame:Hide()
  end
end

--[[
function ShockAndAwe:SetSubFrameBackdrop(subFrame, icon, edgeSize)
  subFrame:SetBackdrop({ bgFile = icon,
      edgeSize = edgeSize
    })
end
]]
function ShockAndAwe:SetSubFrameBackdrop(subFrame, icon, edgeSize)
  subFrame:SetBackdrop({ bgFile = icon,
      tile = false, tileSize = 0, edgeSize = edgeSize

    })
end
